﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;
using System.Collections.Generic;

public class GameManager : MonoBehaviour {

	public Button buttonA;
	public Button buttonB;

	public Text buttonAText;
	public Text buttonBText;
	
	public Text descriptionText;
	
	public Image backgroundImage;
	public Image timer;
	
	public bool timerRunning;
	
	public string textShownOnScreen;
	//public string fullText = "The text you want shown on screen with typewriter effect.";
	
	public int wordsPerSecond = 2; // speed of typewriter
	//private float timeElapsed = 0;  

	public Text whatText;
	public Text do1Text;
	public Text youText;
	public Text do2Text;
	public Text nowText;
	
	public Text gameOverText;
	public Button restartButton;

	public Button startButton;
	public Image logo;
	public Image subline;
	public Image upperPanel;
	public Image lowerPanel;

	public Sprite finalImage;
	public string finalText;
	
	public Button creditsButton;
	public Text creditsText;
	public GridLayoutGroup grid;
	public Image whatImage;
	
	public Text endText;
	public Image blackScreen;
	public Text callToAction;
	public Button htmlLink;
	
	public bool creditsScreen;
	
	public List<Sprite> allBackgrounds = new List<Sprite>();
	public List<Image> startScenes = new List<Image>();

	[System.Serializable]
	public class Scene {
	
		public string name;
		public string descriptionText;
		public string buttonAText;
		public string buttonBText;
		public string decisionA; //name of next scene
		public string decisionB;
		public Sprite image;
		public string failedText;
		public Sprite failedSprite;
	
	}	
	
	public List<Scene> scenes = new List<Scene>();
	
	public Scene currentScene;
	
	public void StartGame() {
	
		OpenScene( "Fly on sandwich" );
		
		HideStartScreen();
	
	}
	
	void Start() {
	
		ShowStartScreen();
	}
	
	public void MakeDecision( int button ) {
	
		StopTimer();
		HideButtons();
		HideQuestion();
		descriptionText.gameObject.SetActive( false );
	
		if ( button == 0 ) {
		
			if ( currentScene.decisionA == "" ) {
		
				FinalScene();
			}
			
			else {
			
				OpenScene( currentScene.decisionA );
			}
		}
		
		else {
		
			
			if ( currentScene.decisionB == "" ) {
				
				FinalScene();
			}
			
			else {
				
				OpenScene( currentScene.decisionB);;
			}
		}
	}
	
	public void OpenScene( string scene ) {
	
		currentScene = FindScene( scene );
	
		buttonAText.text = currentScene.buttonAText;
		buttonBText.text = currentScene.buttonBText;
		//descriptionText.text = currentScene.descriptionText;
		backgroundImage.sprite = currentScene.image;
		
		ShowText( currentScene.descriptionText );
		
	}
	
	public Scene FindScene( string scene ) {
	
		for ( int i = 0; i < scenes.Count; i++ ) {
		
			if ( scenes[i].name == scene ) {
			
				return scenes[i];
			}
		}
	
		return null;
	}
	
	
	public void StartTimer() {
	
		StartCoroutine ( "TimerRoutine" );
	}
	
	IEnumerator TimerRoutine() {
	
		yield return new WaitForSeconds( 2.0f );
	
		timer.gameObject.SetActive(true);
	
		timer.fillAmount = 1.0f;
	
		float timeLoss = 1.0f / (5.0f * 60);
	
		timerRunning = true;

		while (timerRunning == true ) {
		
			timer.fillAmount -= timeLoss;
		
			if ( timer.fillAmount <= 0 ) {
			
				timerRunning = false;
			}
			
			yield return new WaitForSeconds( 0.01f );
		}
		
		GameOver();
	}
	
	
	
	public void ResetTimer() {
	
		timer.fillAmount = 1.0f;
	}
	
	public void GameOver() {
	
		buttonA.gameObject.SetActive( false );
		buttonB.gameObject.SetActive( false );
		HideQuestion();
		
		descriptionText.text = currentScene.failedText;
		
		//ShowText( currentScene.failedText );
		restartButton.gameObject.SetActive( true );
		//gameOverText.gameObject.SetActive( true );
	}
	
	public void ShowText( string text ) {
	
	 	StartCoroutine( TypewriterEffect( text ) );
	 
	}
	
	public void StopTimer() {
	
		StopCoroutine( "TimerRoutine" );
		timer.gameObject.SetActive( false );
	}
	
 	IEnumerator TypewriterEffect( string text ) {
 	
 		//yield return new WaitForSeconds ( 2.0f );
 	
 		descriptionText.gameObject.SetActive( true );
 		descriptionText.text = "";
 		float timeElapsed = 0.0f;
 	
 		while ( descriptionText.text != text ) {
 		
			timeElapsed += 0.05f;
 	
			descriptionText.text = GetWords(text, timeElapsed * wordsPerSecond);
			
			yield return new WaitForSeconds( 0.01f );
		}
		
		AskQuestion();
		StartTimer();
		ShowButtons();
 	}
 	
 	//OK THIS IS A REALLY REALLY BAD HACK
	IEnumerator TypewriterEffect2( string text, Text textAsset ) {
		
		textAsset.gameObject.SetActive( true );
		textAsset.text = "";
		float timeElapsed = 0.0f;
		
		while ( textAsset.text != text ) {
			
			timeElapsed += 0.05f;
			
			textAsset.text = GetWords(text, timeElapsed * wordsPerSecond);
			
			yield return new WaitForSeconds( 0.01f );
		}
	}
	
	private string GetWords(string text, float  wordCount)
	{
		float words = wordCount;
		
		// loop through each character in text
		for (int i = 0; i < text.Length; i++)
		{ 
			if (text[i] == ' ')
			{
				words--;
			}
			
			if (words <= 0)
			{
				return text.Substring(0, i);
			}
		}
		
		return text;
	}
	
	public void AskQuestion() {
	
		/*whatText.gameObject.SetActive( true );
		do1Text.gameObject.SetActive( true );
		youText.gameObject.SetActive( true );
		do2Text.gameObject.SetActive( true );
		nowText.gameObject.SetActive( true );*/
		whatImage.gameObject.SetActive( true );
		
	}
	
	public void ShowButtons() {
	
		buttonA.gameObject.SetActive( true );
		buttonB.gameObject.SetActive( true );
		
		//StartCoroutine( TypewriterEffect2( buttonAText.text, buttonAText ) );
		//StartCoroutine( TypewriterEffect2( buttonBText.text, buttonBText ) );
	}
	
	public void HideButtons() {
	
		buttonA.gameObject.SetActive( false );
		buttonB.gameObject.SetActive( false );
	
	}
	
	public void HideQuestion() {
	
		whatText.gameObject.SetActive( false );
		do1Text.gameObject.SetActive( false );
		youText.gameObject.SetActive( false );
		do2Text.gameObject.SetActive( false );
		nowText.gameObject.SetActive( false );
	
		whatImage.gameObject.SetActive( false );
	}
	
	public void RestartGame() {
	
		OpenScene( "Fly on sandwich" );
		restartButton.gameObject.SetActive( false );
		blackScreen.gameObject.SetActive( false );
		endText.gameObject.SetActive( false );
		callToAction.gameObject.SetActive( false );
		htmlLink.gameObject.SetActive( false );
		//gameOverText.gameObject.SetActive( false );
		
	}
	
	public void ShowStartScreen() {
	
		lowerPanel.gameObject.SetActive( true );
		upperPanel.gameObject.SetActive( true );
		logo.gameObject.SetActive( true );
		lowerPanel.gameObject.SetActive( true );
		upperPanel.gameObject.SetActive( true );
		grid.gameObject.SetActive( true );
		
		StartRandomImagesRoutine();
		StartCoroutine( StartScreenRoutine() );
	}
	
	IEnumerator StartScreenRoutine() {

		yield return new WaitForSeconds( 3.0f);
		
		startButton.gameObject.SetActive( true );
		creditsButton.gameObject.SetActive( true );

		subline.gameObject.SetActive( true );
	}
	
	public void HideStartScreen() {
		
		logo.gameObject.SetActive( false );
		startButton.gameObject.SetActive( false );
		creditsButton.gameObject.SetActive( false );
		lowerPanel.gameObject.SetActive( false );
		upperPanel.gameObject.SetActive( false );
		subline.gameObject.SetActive( false );
		grid.gameObject.SetActive( false );
		
		StopRandomImageRoutine();
		
	}
	
	public void EndGame() {
	
		descriptionText.text = finalText;
		restartButton.gameObject.SetActive( true );
		
	}
	
	public void ShowCredits() {
	
		if ( creditsScreen == false ) {
	
			string credits = 	"Shoe cabinet monster: Art: Miriam Komorek, Text: Benjamin Scharff \n Selfie on Twitter: Julian Hühnermann \n Dreaming Shoe Cabinet: Severin Brettmeister \n Global Game Jam: Thomas Bedenk \n Super Heroes: Christopher Hecht \n Carmack: Matthias Löwe \n Lost Fly: Gregor Haase (FineFin) \n TicTacToe Man: Johannes Kristmann \n Pee: Benjamin Schug \n Coffee Djinn: Art: Miriam Komorek, Text: Benjamin Scharf  \n Impossible Tic Tac Toe: Art: Miriam Komorek, Text: Benjamin Scharf \n Emotional Hero: Marius Winter \n Silent Shoe Cabinet: Markus Bedenk \n Art: Miriam Komorek, Text: Benjamin Scharf \n Alternate Reality: Norbert Haacks \n font: Digital Stripe by www.blambot.com \n Created by Daniel Helbig(@DannyHellfish)"   ;
			creditsText.text = credits;
			creditsButton.gameObject.transform.GetChild( 0).GetComponent<Text>().text = "back to menu";	
			creditsText.gameObject.SetActive( true );
			blackScreen.gameObject.SetActive( true );	
			
			creditsScreen = true;
		}
		
		else {
		
			creditsButton.gameObject.transform.GetChild( 0).GetComponent<Text>().text = "credits";	
			creditsText.gameObject.SetActive( false );
			blackScreen.gameObject.SetActive( false );	
			
			creditsScreen = false;
			
		}
					
	}
	
	public void FinalScene() {
	
		Debug.Log( "Show Final Scene" );
		blackScreen.gameObject.SetActive( true );
		endText.gameObject.SetActive( true );
		restartButton.gameObject.SetActive( true );
		callToAction.gameObject.SetActive( true );
		htmlLink.gameObject.SetActive( true );
	
	}
	
	public void OpenWebsite() {
	
		Application.OpenURL ("http://escalategame.tumblr.com");
	}
	
	public void SendMail() {
	
		
	
	}
	
	public void StartRandomImagesRoutine() {
	
		StartCoroutine( "RandomImage" );
	}
	
	public void StopRandomImageRoutine() {
	
		StopCoroutine( "RandomImage" );
	}
	
	IEnumerator RandomImage() {
	
		bool startScreen = true;
		float randomTime = Random.Range( 0.25f, 0.5f);
		
		while ( startScreen == true ) {
		
			yield return new WaitForSeconds( randomTime );
			
			Sprite randomSprite = allBackgrounds[ Random.Range( 0, allBackgrounds.Count)];
			Image randomImage = startScenes[ Random.Range( 0, startScenes.Count) ];
			
			randomImage.sprite = randomSprite;
			
			randomTime = Random.Range( 0.5f, 1.5f);
		}
	}
	
}
